import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.function.Predicate;

import dao.BFPCalculatorParser;
import dao.BMICalculatorParser;
import dao.BMRCalculatorParser;
import dao.CalorieIntakeCalculatorParser;
import dao.DataImporter;
import dao.Manager;
import dao.PersonImporter;
import dao.PersonParser;
import dto.BFPCalculator;
import dto.BMICalculator;
import dto.BMRCalculator;
import dto.CalorieIntakeCalculator;
import dto.HealthCalculator;
import dto.Person;

//Lab5_1 (KPARK 2019/09/24) Lambda/Stream/Optional/Query & Generic & Package
public class Lab6_1 {
	
	public static Predicate<HealthCalculator> isBMICalculator()	{
	    return c -> c instanceof BMICalculator;
	}

	public static Predicate<HealthCalculator> isBMRCalculator()	{
	    return c -> c instanceof BMRCalculator;
	}

	public static Predicate<HealthCalculator> isBFPCalculator()	{
	    return c -> c instanceof BFPCalculator;
	}

	public static Predicate<HealthCalculator> isCalorieIntakeCalculator()	{
	    return c -> c instanceof CalorieIntakeCalculator;
	}

	public static void main(String[] args) {

		//List<Person> people = PersonImporter.loadCSV("people.csv");
		DataImporter<Person> pdi = new DataImporter<Person>(new PersonParser());
		List<Person> people = pdi.loadCSV("people.csv");
		
		// person manager
		Manager<Person> personManager = new Manager<Person>();
		personManager.addList(people);
		personManager.forEach(System.out::println);
				
		// health manager
		Manager<HealthCalculator> healthManager = new Manager<HealthCalculator>();
		people.forEach(p -> {
			healthManager.add(new BMICalculator(p));
			healthManager.add(new BMRCalculator(p));
			healthManager.add(new BFPCalculator(p));
			healthManager.add(new CalorieIntakeCalculator(p));
		});
		
		List<HealthCalculator> bmiManager = healthManager.getListBy(isBMICalculator());
		System.out.println("\nbmiManager");
		bmiManager.forEach(e -> System.out.println(e));		
		DataImporter<BMICalculator> bmiImporter = new DataImporter<BMICalculator>(new BMICalculatorParser());
		bmiImporter.saveCSV("bmitest.csv", bmiManager);

		List<HealthCalculator> bmrManager = healthManager.getListBy(isBMRCalculator());
		System.out.println("\nbmrManager");
		DataImporter<BMRCalculator> bmrImporter = new DataImporter<BMRCalculator>(new BMRCalculatorParser());
		bmrManager.forEach(e -> System.out.println(e));		
		bmrImporter.saveCSV("bmrtest.csv", bmrManager);
		
		List<HealthCalculator> bfpManager = healthManager.getListBy(isBFPCalculator());
		System.out.println("\nbfpManager");
		bfpManager.forEach(e -> System.out.println(e));		
		DataImporter<BFPCalculator> bfpImporter = new DataImporter<BFPCalculator>(new BFPCalculatorParser());
		bmiImporter.saveCSV("bfptest.csv", bfpManager);

		List<HealthCalculator> ciManager = healthManager.getListBy(isCalorieIntakeCalculator());
		System.out.println("\nciManager");
		DataImporter<CalorieIntakeCalculator> ciImporter = new DataImporter<CalorieIntakeCalculator>(new CalorieIntakeCalculatorParser());
		ciManager.forEach(e -> System.out.println(e));		
		ciImporter.saveCSV("citest.csv", ciManager);

		List<BMICalculator> bmiManager2 = bmiImporter.loadCSV("bmitest.csv");
		System.out.println("\nbmiManager2");
		bmiManager2.forEach(e -> System.out.println(e));		

		List<BMRCalculator> bmrManager2 = bmrImporter.loadCSV("bmrtest.csv");
		System.out.println("\nbmrManager2");
		bmrManager2.forEach(e -> System.out.println(e));	
		
		List<BFPCalculator> bfpManager2 = bfpImporter.loadCSV("bfptest.csv");
		System.out.println("\nbfpManager2");
		bfpManager2.forEach(e -> System.out.println(e));		

		List<CalorieIntakeCalculator> ciManager2 = ciImporter.loadCSV("citest.csv");
		System.out.println("\nciManager2");
		ciManager2.forEach(e -> System.out.println(e));	

		/*
		System.out.println("\npersonManager sorted by age");
		personManager.sort(null);
		personManager.print();
		System.out.println("\npersonManager sorted by activity");
		personManager.sort(new Comparator() {
			@Override
			public int compare(Object o1, Object o2) {
				return ((Person)o1).getActivity().compareTo(((Person)o2).getActivity());
			}
		});
		personManager.print();
		 */		
		//System.out.println("healthManager");
		//healthManager.print();
		
		/*System.out.println("after sort");
		Arrays.sort(people);
		for (var person : people) {
			System.out.println(person);
		}*/
	}

}
